//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----- This is the WeIRCd API interface for modules ----- //
//   Every module for this IRCd MUST include this file      //

#ifndef alloca //makes sure you can use alloca() if you wish to
#define alloca(x) __builtin_alloca(x)
#endif

#include <string.h> //so you can use strlen(), strcpy() and others
#include <stdio.h> //for sprintf() which you may want to use for string processing
#include <stdlib.h> //atoi() etc
#if (HAVE_MALLOC_H != 0)
#include <malloc.h> //for using malloc() or free() if you want to
#endif

// ----- Section (1):  Info Query Functions ----- //
  /* Every module MUST provide a "void module_InfoQuery()"
     function which calls the following functions
     to provide information about itself. The module
     MUST NOT do anything else than calling the
     following functions in its module_InfoQuery()
     routine and afterwards returning program
     execution to WeIRCd. */

void weircd_infoq_ReportName(const char* names);
  /* This MUST be called exactly ONCE, providing the module's name.
     It MUST NOT contain anything else than [a-z][A-Z][' '][0-9]
     and is not allowed to have leading or trailing space chars
     or doubled space chars in between.
     The name MUST NOT be longer than 50 chars or shorter than
     1 char. */

void weircd_infoq_ReportVersion(int major, int minor, int revision);
  /* This MUST be called exactly ONCE, providing the version details */

void weircd_infoq_ReportComment(const char* comment);
  /* A module MAY call this, providing a more in-depth description
     of the module's functionality.
     The comment MUST NOT be longer than 150 chars */

void weircd_infoq_ReportAuthor(const char* name);
  /* A module MAY call this, providing the module's author or
     authors. Seperate multiple names by commas.
     The list of authors MUST NOT be longer than 1500 chars
     and one single author's name MUST NOT be longer than 30
     chars. */

//Important Note:
 /* You might want to use more WeIRCd APIs than this one (e.g.
    the services module api described in weircd_api_services.h). If you do,
    make sure you use the respective weircd_infoq_<apispecificcall>
    function that is described in the API's header file to activate
    it for your module in your module_InfoQuery() function. 

    This is NOT necessary for every api, but just for those that
    enable your module to be loaded differently to just a general
    module (e.g. as a services module). Simply check the respective
    module api header file if it contains such a info query call
    to find out if you need to use one or not to get such
    functionality. */

// ----- Section (2):  Some general info functions ----- //
  /* Those functions give you details about your environment */

const char* weircd_GetDataDir(); //get the data directory where database files etc should be stored

const char* weircd_GetHelpChan(); //the help channel to send users to

const char* weircd_GetNetworkName(); //get the name of the network we are in

unsigned long long weircd_GetCurrentTime(); //get a unix time stamp of the current time

// ----- Section (3):  String functions ----- //
  /* You may use those string functions in your module (you don't
     need to though). You don't need to activate them somehow,
     they are always available. */

void cut(char* vstring, int count);
  /* Cut chars in front of a string away (without resizing the
     string buffer, but by moving any following chars including
     the null terminator more to the front of it) */

int strstw(const char* bigstring,const char* startstring);
  /* Test if the longer string ("bigstring") starts with the
     shorter string. Returns 0 if it does, -1 if it doesn't. */
  
int strendswith(const char* bigstring, const char* endstring);
  /* Checks if the long string ends with the shorter string.
     Returns 0 if it does, -1 if it doesn't. */

void trimleft(char* vstring);
  /* Cut leading space chars, horizontal tabs or also line breaks. */
  
void trimright(char* vstring);
  /* Cut space chars, horizontal tabs and line breaks away from the
     end of the string if there are any. */
  
int charpos(const char* stringle, char letter);
  /* Find the position of the first occurance of a specific letter
     inside a string. Returns a 0-based index (first char is 0,
     second 1, ...). When the letter is not to be found, -1 will
     be returned. */
  
int issimplenumber(const char* s);
  /* Return 0 if the string only contains letters of the range [0..9].
     Return -1 if it doesn't. */
  
void cutonfirstspace(char* vstring);
  /* Cut a string off at the first occurance of a space char */

#ifndef NATIVESTRLWR
void strlwr(char *s);
  /* Lowercase a string, available on Windows natively (at least when compiling with mingw) */
#endif

void addtobuf(char* buf, char byte, unsigned int maxlen);
  /* Add a char to the end of a null-terminated string and preserve proper termination. Will not exceed
     the given maximum buffer len and even when reaching it preserve the null termination. */

void stradd(char* dest, const char* src, int len);
  /* Add one string to another, but, unlike strncat, take an absolute buffer len that both strings need
     to fit into instead of a relative number of bytes to be appended.
     Unlke strncat, this function will also preserve a proper null termination in *all* cases.
     Please note "dest", which is the destination, must be at least len+1 bytes long. */

void strcopy(char* dest, const char* src, int len);
  /* Similar to strncpy but ensures a null termiination. Please ensure dest is actually len+1 bytes long! */
  
// ----- Section (4):  Get info about users ----- //
/* The return value of all those functions is undefined
   when the user doesn't exist (if not specified otherwise) */

int weircd_user_IsOn(const char* user);
 /* Returns 0 if that user exists, otherwise -1 */

int weircd_user_IsInChan(const char* user, const char* channel);
 /* Returns 0 if the channel exists and the user is inside it, otherwise -1 */
 
int weircd_user_IsOpInChan(const char* user, const char* channel);
 /* Returns 0 if the channel exists and the user currently has operator
    status there, otherwise -1 */

int weircd_user_IsIRCOp(const char* user);
 /* Returns 0 if the user is logged in as IRC op */

int weircd_user_IsServicesOp(const char* user);
 /* Returns 0 if the user is a logged in IRC op with services admin flag */

int weircd_user_GetTimeSinceNickChange(const char* user);
 /* Returns the amount of seconds that has passed since the current nick
    was obtained by the user */

const char* weircd_user_GetLoginAccountName(const char* user);
 /* If the user is logged in, returns his/her account name (set by services).
    Otherwise, returns NULL. */

const char* weircd_user_GetIp(const char* user);
 /* Returns the user's ip in readable format */
 
// ----- Section (5):  Get info about channels ----- //
/* The return value of all those functions is undefined
   when the channel doesn't exist (if not specified otherwise) */

int weircd_channel_Exists(const char* channel);
 /* Returns 0 if channel is formed, otherwise -1 */
 
const char* weircd_channel_GetTopic(const char* channel);
 /* Returns channel topic or NULL if not set */